import { useSyncExternalStore } from 'use-sync-external-store/shim';

var __DEV__ = process.env.NODE_ENV !== 'production';
var isInAction = false;
var run = function (fn) {
    fn();
};
var resso = function (obj) {
    if (__DEV__ && Object.prototype.toString.call(obj) !== '[object Object]') {
        throw new Error('object required');
    }
    var state = {};
    var actions = {};
    Object.keys(obj).forEach(function (key) {
        var initVal = obj[key];
        if (initVal instanceof Function) {
            actions[key] = function () {
                var args = [];
                for (var _i = 0; _i < arguments.length; _i++) {
                    args[_i] = arguments[_i];
                }
                isInAction = true;
                var res = initVal.apply(void 0, args);
                isInAction = false;
                return res;
            };
            return;
        }
        var listeners = new Set();
        state[key] = {
            subscribe: function (listener) {
                listeners.add(listener);
                return function () { return listeners.delete(listener); };
            },
            getSnapshot: function () { return obj[key]; },
            setSnapshot: function (val) {
                if (val !== obj[key]) {
                    obj[key] = val;
                    run(function () { return listeners.forEach(function (listener) { return listener(); }); });
                }
            },
            useSnapshot: function () {
                return useSyncExternalStore(state[key].subscribe, state[key].getSnapshot);
            },
        };
    });
    var setState = function (key, val) {
        if (key in obj) {
            if (key in state) {
                var newVal = val instanceof Function ? val(obj[key]) : val;
                state[key].setSnapshot(newVal);
            }
            else if (__DEV__) {
                throw new Error("'".concat(key, "' is a function, can not update"));
            }
        }
        else if (__DEV__) {
            throw new Error("'".concat(key, "' is not initialized in store"));
        }
    };
    return new Proxy((function () { return undefined; }), {
        get: function (_, key) {
            if (key in actions) {
                return actions[key];
            }
            if (isInAction) {
                return obj[key];
            }
            try {
                return state[key].useSnapshot();
            }
            catch (err) {
                return obj[key];
            }
        },
        set: function (_, key, val) {
            setState(key, val);
            return true;
        },
        apply: function (_, __, _a) {
            var key = _a[0], updater = _a[1];
            setState(key, updater);
        },
    });
};
resso.config = function (_a) {
    var batch = _a.batch;
    run = batch;
};

export { resso as default };
